(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

open Formulas

exception Exc_assertion of string

type assertion =
    string * string list * Formulas.formula * Formulas.formula

let print_assertion showtypes oc 
    (protocol_filename, public_symbols, initial_formula, safety_formula) =
  let os = output_string oc
  in
    os "uses\n  \"";
    os protocol_filename;
    os "\"\n\npublic\n  ";
    os (Utils.string_of_string_list "," public_symbols);
    os "\n\ninitial\n  ";
    os (string_of_formula showtypes initial_formula);
    os "\n\nsafety\n  ";
    os (string_of_formula showtypes safety_formula);
    os "\n"

let check_assertion_fragment (specname,publics,initf,alwaysf) =
  if not (is_pi1 initf)
  then raise (Exc_assertion( "the initial formula is not in the required " ^
			    "syntactic fragment of the logic"));
  if not (is_safety_formula alwaysf)
  then raise (Exc_assertion( "the second formula is not in the required " ^
			    "syntactic fragment of the logic"))

