(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

open Expressions

type trs = ( expression * expression ) list

let trs_to_string =
  List.fold_left 
    (fun s (l,r) -> s ^
       (string_of_expression false l) ^ " -> " ^
       (string_of_expression false r) ^ "\n")
    ""

type substitution = (string * expression) list
let string_of_substitution sigma =
  "[" ^
    (Utils.string_of_string_list "; "
       (List.map
	  (function (var,term) ->
	     var ^ " |-> " ^ (string_of_expression false term))
	  sigma)) ^
    "]"

let rec apply_subst subst = function
    Ident (info,s) -> 
      (try List.assoc s subst
       with
	   Not_found -> Ident (info,s))
  | Int (info,i) -> Int (info,i)
  | Apply(info,f,tl) -> Apply(info,f,List.map (apply_subst subst) tl)
  | Infix(info,op,e1,e2) -> Infix(info,op,
			     (apply_subst subst e1),
			     (apply_subst subst e2))
  | Inverse (info,e) -> Inverse (info,apply_subst subst e)
  | Tuple (info,el) -> Tuple(info,List.map (apply_subst subst) el)
  | Liste (info,el) -> Liste(info,List.map (apply_subst subst) el)
  | AssocList (info,pl) ->
      AssocList(info,List.map
		  (function (e1,e2) -> (apply_subst subst e1,
					apply_subst subst e2))
		  pl)





